<?php

namespace Drupal\views_sql_twig_relationship\Plugin\views\relationship;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\relationship\EntityReverse;
use Drupal\views_sql_twig_fields\TwigSqlFields;
use Drupal\views\Views;

/**
 * SQL Twig relationship entity reverse handler .
 *
 * @ingroup views_relationship_handlers
 *
 * @ViewsRelationship("views_sql_twig_relationship_reverse")
 */
class ViewsSqlTwigRelationshipReverse extends EntityReverse {
  use TwigSqlFields;

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $this->twigSqlDefineOptions($options);
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsform($form, $form_state);

    $this->twigSqlBuildOptionsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    $this->ensureMyTable();
    $def = $this->definition;
    $this->regenerateSqlQuery();

    // Figure out what base table this relationship brings to the party.
    $table_data = Views::viewsData()->get($this->definition['base']);
    $base_field = empty($this->definition['base field']) ? $table_data['table']['base']['field'] : $this->definition['base field'];

    $first = [
      'sql_condition' => $this->options['sql'],
      'left_table' => $this->tableAlias,
      'left_field' => $left_field,
      'table' => $def['field table'],
      'field' => $def['field field'],
      'adjusted' => TRUE,
    ];

    if (!empty($this->options['required'])) {
      $firt['type'] = 'INNER';
    }

    if (!empty($this->definition['extra'])) {
      $first['extra'] = $def['extra'];
    }

    $first_join = $this->joinManager->createInstance('twig_sql_join', $first);

    $this->first_alias = $this->query->addTable($this->definition['field table'], $this->relationship, $first_join);

    // Second, relate the field table to the entity specified using
    // the entity id on the field table and the entity's id field.
    $second = [
      'left_table' => $this->first_alias,
      'left_field' => 'entity_id',
      'table' => $def['base'],
      'field' => $def['base field'],
      'adjusted' => TRUE,
    ];

    if (!empty($this->options['required'])) {
      $second['type'] = 'INNER';
    }

    $second_join = $this->joinManager->createInstance('twig_sql_join', $second);
    $second_join->adjusted = TRUE;

    // Use a short alias for this:
    $alias = $this->definition['field_name'] . '_' . $this->table;

    $this->alias = $this->query->addRelationship($alias, $second_join, $this->definition['base'], $this->relationship);
  }

}
