<?php

namespace Drupal\views_sql_twig_fields\Plugin\views\relationship;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\relationship\Standard;
use Drupal\views_sql_twig_fields\TwigSqlFields;
use Drupal\views\Views;

/**
 * SQL Twig relationship handler .
 *
 * @ingroup views_relationship_handlers
 *
 * @ViewsRelationship("views_sql_twig_fields_relationship")
 */
class ViewsSqlTwigFieldsRelationship extends Standard {
  use TwigSqlFields {
    submitOptionsForm as twigSqlSubmitOptionsForm;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    if (empty($this->definition['label'])) {
      $this->definition['label'] = 'Global SQL Twig';
    }
    $options = parent::defineOptions();
    $this->twigSqlDefineOptions($options);

    // Change admin_label default.
    $options['admin_label'] = ['default' => 'Global SQL Twig'];
    $options['twig table'] = ['default' => 'node_field_data'];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsform($form, $form_state);

    $tables = \Drupal::service('database')->schema()->findTables('%');

    $form['twig_table'] = [
      '#type' => "select",
      '#title' => t('Twig SQL Base Table'),
      '#description' => t('This should be a valid SQL table name.'),
      '#default_value' => $this->options['twig table'],
      '#options' => $tables,
      '#weight' => -100,
      "#disabled" => $this->notAdmin(),
      '#required' => TRUE,
    ];

    $this->twigSqlBuildOptionsForm($form, $form_state);

    // Hide variables that are not available.
    unset($form['twig_details']['twig_definition']['variables']);
  }

  /**
   * {@inheritdoc}
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    $this->twigSqlSubmitOptionsForm($form, $form_state);

    $this->options['twig table'] =
      $form_state->getValue(['options', 'twig_table']);
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    $table = $this->options['twig table'];
    $this->definition['base'] = $table;
    $this->ensureMyTable();
    $this->regenerateSqlQuery();
    $def = $this->definition;

    // Figure out what base table this relationship brings to the party.
    $table_data = Views::viewsData()->get($table);

    $def['sql_condition'] = $this->options['sql'];
    $def['adjusted'] = TRUE;
    $def['table'] = $table;

    // These fields are not used.
    $def['field'] = $this->field;
    $def['left_table'] = $this->tableAlias;
    $def['left_field'] = $this->realField;

    if (!empty($this->options['required'])) {
      $def['type'] = 'INNER';
    }

    if (!empty($this->definition['extra'])) {
      $def['extra'] = $this->definition['extra'];
    }

    $join = Views::pluginManager('join')->createInstance('twig_sql_join', $def);

    // Use a short alias for this:
    $alias = $def['table'];

    $this->alias = $this->query->addRelationship($alias, $join, $table, $this->relationship);

    // Add access tags if the base table provide it.
    if (empty($this->query->options['disable_sql_rewrite']) && isset($table_data['table']['base']['access query tag'])) {
      $access_tag = $table_data['table']['base']['access query tag'];
      $this->query->addTag($access_tag);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityType() {
    return $this->options['twig entity'];
  }

}
